%**********************************************************
% Example 5 (ex05.m)                                      *
%                                                         *
% Calculation of the influence of composition on the      *
% A3 temperature in an Fe-Cr-C alloy.                     * 
%                                                         *
% The A3 temperature is calculated for a large number of  *
% uniformly distributed compositions in composition space.*
% The carbon content belongs to the interval              *
% [1E-4:5E-3] (weight fraction) and the cromium content   *
% belongs to the interval [1E-2,3E-2].                    *
%                                                         *
% The relative frequency (the fraction) of compositions   *
% belonging to a certain A3 temperature interval is then  *
% plotted.                                                *         
%**********************************************************
% Initiate the TC subsystem
tc_init_root;

% Choose database
tc_open_database('FEDEMO');

% Check for errors
tc_check_error;

% Select elements and retrieve data
sel='fe c cr';
tc_element_select(sel);
tc_get_data;

% Defines intervals for the composition
Nc=15;  % number of points in C (discretizations)
Ncr=15; % number of points in Cr (discretizations)
cmin=1E-4;  % min C content
cmax=5E-3;   % max C content
crmin=1E-2; % min Cr content
crmax=3E-2;  % max Cr content
c_intv=linspace(cmin,cmax,Nc);
cr_intv=linspace(crmin,crmax,Ncr);

% Define arrays for storing the calculation results
A3_temp=zeros(Ncr,Nc);

% Set conditions and make an equilibrium calculation
tc_set_condition('t',1100);
tc_set_condition('p',101325);
tc_set_condition('n',1);
tc_set_condition('w(c)',5E-3);
tc_set_condition('w(cr)',1.5e-2);
tc_compute_equilibrium;
tc_check_error;

% To calculate the A3 of liquidus temperature delete 
% the temperature condition and fix one fas
tc_delete_condition('t');
Nproc=0;
for n=1:Ncr
    tc_set_condition('w(cr)',cr_intv(n));  
    for k=1:Nc
        tc_set_condition('w(c)',c_intv(k));
        % To calculate the A3 temperature 
        % fix the bcc phase with value 0
        tc_set_phase_status('bcc','fixed',0);
        tc_compute_equilibrium;
        tc_check_error;
        A3_temp(n,k)=tc_get_value('t');
    end
    Nproc=Nproc+100/Ncr;
    disp([num2str(Nproc),'% of the calculations completed'])
end

% Plot the result with Nbars number of bars.
Nbars=12;
A3_vect=reshape(A3_temp,Ncr*Nc,1)-273.15;
Tint=linspace(min(A3_vect),max(A3_vect),Nbars);
b1=bar(Tint,hist(A3_vect,Nbars)./(Ncr*Nc));
xlabel('Temperature [C]')
ylabel('Relative frequency')

% set title
title(['Variation of A3 temperature, w(c)\in[',num2str(cmin),...
      ',',num2str(cmax),'], w(cr)\in','[',num2str(crmin)...
      ,',',num2str(crmax),']'])        

% add text
cord_x=0.6*max(get(b1,'XData'))+0.4*min(get(b1,'XData'));
cord_y=0.8*max(get(b1,'YData'))+0.2*min(get(b1,'YData'));
cq=cell(5,1);
cq(1)={['mean value = ',num2str(mean(A3_vect)),'C']};
cq(2)={['median = ',num2str(median(A3_vect)),'C']};
cq(3)={['min = ',num2str(min(A3_vect)),'C']};
cq(4)={['max = ',num2str(max(A3_vect)),'C']};
cq(5)={['\sigma = ',num2str(std(A3_vect))]};
text(cord_x,cord_y,cq);