%**********************************************************
% Example 6: ex06_interfacial_energy.m                    *
%                                                         *
% Calculation of interfacial energy between BCC and M7C3  *
% for a Fe-12Cr-0.1C alloy                                *
%                                                         *
%**********************************************************


clear variables;
clc;
tc_init_root;
database='FEDEMO'; 

components={'FE', 'C', 'CR'};
dependentComponent = 'FE';
matrix_phase='BCC_A2';
precipitate_phase='M7C3'; 
T=1053;   

% First perform an equilibrium calculation to get the inputs to the 
% interfacial energy model

tc_define_system(database, components, {'*'}, cellstr([matrix_phase ' ' precipitate_phase]))

tc_set_condition('T', T);
tc_set_condition('n', 1);
tc_set_condition('p', 1.01325e5);
tc_set_condition('w(c)',  0.1e-2);  
tc_set_condition('w(cr)', 12.0e-2);  
tc_compute_equilibrium;

[a,b]=tc_error;
if a~=0
    s=sprintf(' ERROR %d',a);
    disp(s);
    disp(b);
    return;
end


disp('phases:');
[~, phases]=tc_list_phase
disp('conditions:');
[~, conditions]=tc_list_conditions

% 'tc_list_component' retreives the selected components plus special
% components that represent the electron and vacancies (/- ZE VA). 
% We need to filter out the special components and only keep 
% non-special components.
[~, all_components]=tc_list_component 
indexes_non_special_components=cell2mat(cellfun(@(x) isempty(x), regexpi(all_components, '^VA|ZE|/-$', 'start'), 'UniformOutput', false));
non_special_components=all_components(indexes_non_special_components)

indexes_non_zero_volume_components=cell2mat(cellfun(@(x) isempty(x), regexpi(non_special_components, '^[C|N]$', 'start'), 'UniformOutput', false))

% Convert to u-fraction by:
% 1. Set the elements that don't contribute to volume as special. 
% 2. Reset the compositions for non-dependent components in mole-fractions  
for k=1:length(non_special_components)
    component = non_special_components{k};
    if (indexes_non_zero_volume_components(k) == 0)
        disp('Zero volume component:') 
        component
        tc_poly3_command(['change-status component ' component '=special'])
    end
    if ~(strcmpi(component, dependentComponent))
        massFractionCondition = ['W(' component ')'];
        moleFractionCondition = ['X(' component ')'];
        moleFraction = tc_get_value(moleFractionCondition);
        tc_delete_condition(massFractionCondition)
        tc_set_condition(moleFractionCondition, moleFraction)
    end    
end

tc_compute_equilibrium;
[a,b]=tc_error;
if a~=0
    s=sprintf(' ERROR %d',a);
    disp(s);
    disp(b);
    return;
end

% Mole-fraction for a component now corresponds to u-fraction 
% since zero-volume components are set to special 
u_fraction = zeros(1, length(non_special_components));
for k=1:length(non_special_components)
    component = non_special_components{k};
    u_fraction(k) = tc_get_value(['X(' component ')']);
end

% Molar volume for phase is now counted per amount of volume contributing
% components
vol_matrix=tc_get_value(['vm(' matrix_phase ')'])
vol_precip=tc_get_value(['vm(' precipitate_phase ')'])

% Get the index of the dependent component in the non_special_components array
index_dependent_component=find(strcmpi(non_special_components, 'Fe')==1) 

% Create a new equilibrium in POLY-3 since tc_get_surface_energy will remove
% existing conditions 
tc_create_new_equilibrium(2)

% TC_GET_SURFACE_ENERGY
% input arguments:	string with matrix phase name
%			        string with precipitate phase name
%               	integer index of dependent component in u-fractions
% 			        double with the temperature
%                   array of doubles containing the u-fractions
% 			        double with the volume of the matrix phase
% 			        double with the volume of the precipitating phase
% output arguments:	the surface energy
disp(['The interfacial energy ' matrix_phase '/' precipitate_phase ' [J/m^2] is computed as:'])

sigma=tc_get_surface_energy(matrix_phase, precipitate_phase, index_dependent_component, T, u_fraction, vol_matrix, vol_precip)

disp('The initial conditions are kept in eq. 1:');
tc_select_equilibrium(1);
[~, conditions]=tc_list_conditions

tc_deinit;

